@@ -1,15 +1,17 @@
 import os
 from os import path
 
-import tqdm
-
+from typing import Optional, NamedTuple
 import tensorflow as tf
-from tensorflow.keras.optimizers import Adam
-from tensorflow.keras.layers import Input, Dense, Dropout
+import tqdm
 from tensorflow.keras import Model, initializers
+from tensorflow.keras.layers import Dense, Dropout, Input
+from tensorflow.keras.optimizers import Adam
 
 from ydata_synthetic.synthesizers.gan import BaseModel
-from ydata_synthetic.synthesizers.loss import gradient_penalty
+from ydata_synthetic.synthesizers.loss import Mode, gradient_penalty
+from ydata_synthetic.utils.gumbel_softmax import GumbelSoftmaxActivation
+
 
 class DRAGAN(BaseModel):
 
@@ -21,10 +23,11 @@ class DRAGAN(BaseModel):
         self.gradient_penalty_weight = gradient_penalty_weight
         super().__init__(model_parameters)
 
-    def define_gan(self):
+    def define_gan(self, col_transform_info: Optional[NamedTuple] = None):
         # define generator/discriminator
         self.generator = Generator(self.batch_size). \
-            build_model(input_shape=(self.noise_dim,), dim=self.layers_dim, data_dim=self.data_dim)
+            build_model(input_shape=(self.noise_dim,), dim=self.layers_dim, data_dim=self.data_dim,
+                        activation_info=col_transform_info)
 
         self.discriminator = Discriminator(self.batch_size). \
             build_model(input_shape=(self.data_dim,), dim=self.layers_dim)
@@ -33,7 +36,7 @@ class DRAGAN(BaseModel):
         self.d_optimizer = Adam(self.d_lr, beta_1=self.beta_1, beta_2=self.beta_2, clipvalue=0.001)
 
     def gradient_penalty(self, real, fake):
-        gp = gradient_penalty(self.discriminator, real, fake, mode='dragan')
+        gp = gradient_penalty(self.discriminator, real, fake, mode= Mode.DRAGAN)
         return gp
 
     def update_gradients(self, x):
@@ -113,8 +116,21 @@ class DRAGAN(BaseModel):
         d_loss, g_loss = self.update_gradients(train_data)
         return d_loss, g_loss
 
-    def train(self, data, train_arguments):
-        train_loader = self.get_data_batch(data, self.batch_size)
+    def train(self, data, train_arguments, num_cols, cat_cols):
+        """
+        Args:
+            data: A pandas DataFrame or a Numpy array with the data to be synthesized
+            train_arguments: GAN training arguments.
+            num_cols: List of columns of the data object to be handled as numerical
+            cat_cols: List of columns of the data object to be handled as categorical
+        """
+        super().train(data, num_cols, cat_cols)
+
+        processed_data = self.processor.transform(data)
+        self.data_dim = processed_data.shape[1]
+        self.define_gan(self.processor.col_transform_info)
+
+        train_loader = self.get_data_batch(processed_data, self.batch_size)
 
         # Create a summary file
         train_summary_writer = tf.summary.create_file_writer(path.join('..\dragan_test', 'summaries', 'train'))
@@ -125,47 +141,48 @@ class DRAGAN(BaseModel):
                     batch_data = tf.cast(batch_data, dtype=tf.float32)
                     d_loss, g_loss = self.train_step(batch_data)
 
-            print(
-                "Epoch: {} | disc_loss: {} | gen_loss: {}".format(
-                    epoch, d_loss, g_loss
-                ))
+                print(
+                    "Epoch: {} | disc_loss: {} | gen_loss: {}".format(
+                        epoch, d_loss, g_loss
+                    ))
 
-            if epoch % train_arguments.sample_interval == 0:
-                # Test here data generation step
-                # save model checkpoints
-                if path.exists('./cache') is False:
-                    os.mkdir('./cache')
-                model_checkpoint_base_name = './cache/' + train_arguments.cache_prefix + '_{}_model_weights_step_{}.h5'
-                self.generator.save_weights(model_checkpoint_base_name.format('generator', epoch))
-                self.discriminator.save_weights(model_checkpoint_base_name.format('discriminator', epoch))
+                if epoch % train_arguments.sample_interval == 0:
+                    # Test here data generation step
+                    # save model checkpoints
+                    if path.exists('./cache') is False:
+                        os.mkdir('./cache')
+                    model_checkpoint_base_name = './cache/' + train_arguments.cache_prefix + '_{}_model_weights_step_{}.h5'
+                    self.generator.save_weights(model_checkpoint_base_name.format('generator', epoch))
+                    self.discriminator.save_weights(model_checkpoint_base_name.format('discriminator', epoch))
 
             self.g_optimizer=self.g_optimizer.get_config()
             self.d_optimizer=self.d_optimizer.get_config()
 
 
 class Discriminator(Model):
-  def __init__(self, batch_size):
-    self.batch_size = batch_size
+    def __init__(self, batch_size):
+        self.batch_size = batch_size
 
-  def build_model(self, input_shape, dim):
-    input = Input(shape=input_shape, batch_size=self.batch_size)
-    x = Dense(dim * 4, kernel_initializer=initializers.TruncatedNormal(mean=0., stddev=0.5), activation='relu')(input)
-    x = Dropout(0.1)(x)
-    x = Dense(dim * 2, activation='relu')(x)
-    x = Dropout(0.1)(x)
-    x = Dense(dim, activation='relu')(x)
-    x = Dense(1, activation='sigmoid')(x)
-    return Model(inputs=input, outputs=x)
+    def build_model(self, input_shape, dim):
+        input = Input(shape=input_shape, batch_size=self.batch_size)
+        x = Dense(dim * 4, kernel_initializer=initializers.TruncatedNormal(mean=0., stddev=0.5), activation='relu')(input)
+        x = Dropout(0.1)(x)
+        x = Dense(dim * 2, activation='relu')(x)
+        x = Dropout(0.1)(x)
+        x = Dense(dim, activation='relu')(x)
+        x = Dense(1, activation='sigmoid')(x)
+        return Model(inputs=input, outputs=x)
 
 class Generator(Model):
-  def __init__(self, batch_size):
-    self.batch_size = batch_size
-
-  def build_model(self, input_shape, dim, data_dim):
-    input = Input(shape=input_shape, batch_size = self.batch_size)
-    x = Dense(dim, kernel_initializer=initializers.TruncatedNormal(mean=0., stddev=0.5), activation='relu')(input)
-    x = Dense(dim * 2, activation='relu')(x)
-    x = Dense(dim * 4, activation='relu')(x)
-    x = Dense(data_dim)(x)
-    return Model(inputs=input, outputs=x)
+    def __init__(self, batch_size):
+        self.batch_size = batch_size
 
+    def build_model(self, input_shape, dim, data_dim, activation_info: NamedTuple = None):
+        input = Input(shape=input_shape, batch_size = self.batch_size)
+        x = Dense(dim, kernel_initializer=initializers.TruncatedNormal(mean=0., stddev=0.5), activation='relu')(input)
+        x = Dense(dim * 2, activation='relu')(x)
+        x = Dense(dim * 4, activation='relu')(x)
+        x = Dense(data_dim)(x)
+        if activation_info:
+            x = GumbelSoftmaxActivation(activation_info)(x)
+        return Model(inputs=input, outputs=x)
