@@ -1,10 +1,12 @@
 import os
 from os import path
 import numpy as np
+from typing import List, Optional, NamedTuple
 from tqdm import trange
 
 from ydata_synthetic.synthesizers.gan import BaseModel
 from ydata_synthetic.synthesizers import TrainParameters
+from ydata_synthetic.utils.gumbel_softmax import GumbelSoftmaxActivation
 
 import tensorflow as tf
 from tensorflow.keras.layers import Input, Dense, Dropout
@@ -18,9 +20,10 @@ class VanilllaGAN(BaseModel):
     def __init__(self, model_parameters):
         super().__init__(model_parameters)
 
-    def define_gan(self):
+    def define_gan(self, activation_info: Optional[NamedTuple]):
         self.generator = Generator(self.batch_size).\
-            build_model(input_shape=(self.noise_dim,), dim=self.layers_dim, data_dim=self.data_dim)
+            build_model(input_shape=(self.noise_dim,), dim=self.layers_dim, data_dim=self.data_dim,
+                        activation_info = activation_info)
 
         self.discriminator = Discriminator(self.batch_size).\
             build_model(input_shape=(self.data_dim,), dim=self.layers_dim)
@@ -58,14 +61,24 @@ class VanilllaGAN(BaseModel):
         stop_i = start_i + batch_size
         shuffle_seed = (batch_size * seed) // len(train)
         np.random.seed(shuffle_seed)
-        train_ix = np.random.choice(list(train.index), replace=False, size=len(train))  # wasteful to shuffle every time
+        train_ix = np.random.choice(train.shape[0], replace=False, size=len(train))  # wasteful to shuffle every time
         train_ix = list(train_ix) + list(train_ix)  # duplicate to cover ranges past the end of the set
-        x = train.loc[train_ix[start_i: stop_i]].values
-        return np.reshape(x, (batch_size, -1))
+        return train[train_ix[start_i: stop_i]]
+
+    def train(self, data, train_arguments: TrainParameters, num_cols: List[str], cat_cols: List[str]):
+        """
+        Args:
+            data: A pandas DataFrame or a Numpy array with the data to be synthesized
+            train_arguments: GAN training arguments.
+            num_cols: List of columns of the data object to be handled as numerical
+            cat_cols: List of columns of the data object to be handled as categorical
+        """
+        super().train(data, num_cols, cat_cols)
+
+        processed_data = self.processor.transform(data)
+        self.data_dim = processed_data.shape[1]
+        self.define_gan(self.processor.col_transform_info)
 
-    def train(self,
-              data,
-              train_arguments: TrainParameters):
         iterations = int(abs(data.shape[0]/self.batch_size)+1)
 
         # Adversarial ground truths
@@ -77,7 +90,7 @@ class VanilllaGAN(BaseModel):
                 # ---------------------
                 #  Train Discriminator
                 # ---------------------
-                batch_data = self.get_data_batch(data, self.batch_size)
+                batch_data = self.get_data_batch(processed_data, self.batch_size)
                 noise = tf.random.normal((self.batch_size, self.noise_dim))
 
                 # Generate a batch of events
@@ -118,12 +131,14 @@ class Generator(tf.keras.Model):
     def __init__(self, batch_size):
         self.batch_size=batch_size
 
-    def build_model(self, input_shape, dim, data_dim):
+    def build_model(self, input_shape, dim, data_dim, activation_info: Optional[NamedTuple] = None):
         input= Input(shape=input_shape, batch_size=self.batch_size)
         x = Dense(dim, activation='relu')(input)
         x = Dense(dim * 2, activation='relu')(x)
         x = Dense(dim * 4, activation='relu')(x)
         x = Dense(data_dim)(x)
+        if activation_info:
+            x = GumbelSoftmaxActivation(activation_info)(x)
         return Model(inputs=input, outputs=x)
 
 class Discriminator(tf.keras.Model):
@@ -138,4 +153,4 @@ class Discriminator(tf.keras.Model):
         x = Dropout(0.1)(x)
         x = Dense(dim, activation='relu')(x)
         x = Dense(1, activation='sigmoid')(x)
-        return Model(inputs=input, outputs=x)
\ No newline at end of file
+        return Model(inputs=input, outputs=x)
